/*
 Simple navigation of PropBot using stationary Ping
*/

#include "simpletools.h"                  // Include simpletools lib
#include "servo.h"                        // Include servo lib
#include "ping.h"                         // Include ping lib

int turretPin = 14;                       // Standard servo connected to P14
int pingPin = 15;                         // Ping connected to P15
int piezo = 4;                            // Piezo connected to P4  

int main() {                              // Main function
  
  // Make sound at startup
  freqout(piezo, 200, 500);
  freqout(piezo, 200, 1000);
  freqout(piezo, 200, 2000);
  
  // Be sure turret is centered  
  // First value is position, second is delay for servo transit
  doPing(900, 1000);
  
  while(1) {                               // Start endless loop
  
    if (doPingSimple() <5) {  		 // Back up and turn if distance
      robot_reverse();				 //  is less than 5 inches
      pause(600);
      robot_turn_right();
      pause(1000);
    }

    robot_forward();
    pause(100);
     
  }
  
}

// Motor control functions
void robot_forward(void) {
  servo_angle(16, 1800);
  servo_angle(17, 0);
}  
void robot_reverse(void) {
  servo_angle(16, 0);
  servo_angle(17, 1800);
}  
void robot_turn_left(void) {
  servo_angle(16, 1800); 
  servo_angle(17, 1800);                     
}  
void robot_turn_right(void) {
  servo_angle(16, 0);
  servo_angle(17, 0);
}

// Rotate turret, fire Ping, return distance
int doPing(int turretPos, int waitDelay) {
  servo_angle(turretPin, turretPos);  // Move turret
  pause(500);                         // Transit delay
  int Dist = ping_inches(pingPin);    // Get object distance
  print("Dist = %d\n", Dist);         // Display distance on Terminal
  pause(waitDelay);                   // Wait before moving again
  return Dist;                        // Return distance from function
}

// Same as above, but only fire Ping
int doPingSimple(void) {
  int Dist = ping_inches(pingPin);    // Get object distance
  print("Dist = %d\n", Dist);         // Display distance on Terminal
  return Dist;                        // Return distance from function
}