/*
  Wander looking for objects, and move away or closer to them
  depending on distances reported by Ping sensor
*/

#include "simpletools.h"                  // Include simpletools lib
#include "servo.h"                        // Include servo lib
#include "ping.h"                         // Include ping lib

void watcher();                           // Forward declaration for background timer

int turretPin = 14;                       // Standard servo connected to P14
int pingPin = 15;                         // Ping connected to P15
int piezo = 4;                            // Piezo connected to P4 
int minDistance = 5;                      // Minimum distance (inches) allowed
int watchDog;                             // Re-used variable for cog ID

int main() {                              // Main function
  
  // Sound at startup
  freqout(piezo, 200, 500);
  freqout(piezo, 200, 1000);
  freqout(piezo, 200, 2000);

  // Turret positioning values  
  int turretCenter = 900;
  int turretRight = 400;
  int turretLeft = 1400;
  
  print ("Started\n");                     // Send text to Terminal; for debuging
  watchDog = cog_run(&watcher, 10);        // Run background timer in another cog  
  
  doPing(turretCenter, 1000);              // Set turret to look forward
  
  while(1) {                               // Start endless loop
  
    if (doPingSimple() < minDistance) {    // Check if object immediately ahead
      robot_reverse();                     // Move away and go in different direction
      pause(600);
      robot_turn_right();
      pause(1000);
    }

    robot_forward();                       // Resume forward movement
    pause(100);                            // Slight delay
      
    if (input(26)) {                       // Check if LED on P26 is active (timer expired)
      cog_end(watchDog);                   // Stop background timer; free resources
      low(26);                             // Turn P26 LED back off
      robot_stop();                        // Stop robot
      int rightPing = doPing(turretRight, 500); // Scan right and left for objects
      int leftPing = doPing(turretLeft, 500);      
      print("Right = %d\n", rightPing);    // Display distance values in Terminal
      print("Left = %d\n", leftPing);
      if (rightPing <= minDistance) {      // Reset distances if too close
        rightPing = 150;
      }        
      if (leftPing < minDistance) {
        leftPing = 150;
      }      
      if (rightPing <= leftPing) {         // Steer right or left depending on proximity
        robot_turn_left();
      } else {
        robot_turn_right();
      }
      doPing(turretCenter, 10);           // Reset turret to center
      pause(700);                         
      watchDog = cog_run(&watcher, 10);   // Re-run background timer
    }      
     
  }
  
}

// Rotate turret, fire Ping, return distance
int doPing(int turretPos, int waitDelay) {
  servo_angle(turretPin, turretPos);  // Move turret
  pause(500);                         // Transit delay
  int Dist = ping_inches(pingPin);    // Get object distance
  pause(waitDelay);                   // Wait before moving again
  return Dist;                        // Return distance from function
}

// Fire Ping only
int doPingSimple(void) {
  int Dist = ping_inches(pingPin);    // Get object distance
  return Dist;                        // Return distance from function
}

// Set background timer in a separate cog
void watcher() {
  pause (4000);                       // Wait 4 seconds
  high(26);                           // Turn on P26 (signals timer has expired)
}  

// Motor functions
void robot_forward(void) {
  servo_angle(16, 1800);
  servo_angle(17, 0);
}  
void robot_reverse(void) {
  servo_angle(16, 0);
  servo_angle(17, 1800);
}  
void robot_turn_left(void) {
  servo_angle(16, 1800); 
  servo_angle(17, 1800);                     
}  
void robot_turn_right(void) {
  servo_angle(16, 0);
  servo_angle(17, 0);
}

void robot_stop(void) {
  servo_angle(16, 900);
  servo_angle(17, 900);
}